/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Aver TVPhone95 remote control driver. WORKING !!!!!!
              Version 3.3 - JTVRemote Spec 1.2

 Prototype (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// BTTool.exe display for "AverMedia TVPhone (95)":
// GPIO_Dma_Ctl - 00000004;
// GPIO_Out_En  - 00007000;
// GPIO_Data    - remarked column (for pressed keys and closed JTV-window).
// When open JTV-window, GPIO_Data values changes from "x" to "c", 
// where "x" - random value (ex: 00057fxx-00057fxc).

KeyDef keys[]=
{ 
        { 0x6d, "Key_TV", "" },
        { 0xad, "Key_FM", "" },

        { 0x4d, "Key_Power", "Power", },

        { 0x35, "Key_Display", "", },
        { 0x2d, "Key_Freeze", "", },
        { 0x8d, "Key_Capture", "", },
        { 0x3d, "Key_Video", "", },

        { 0x85, "Key_1", "1", },
        { 0x45, "Key_2", "2", },
        { 0xc5, "Key_3", "3", },
        { 0x1d, "Key_Audio", },

        { 0x25, "Key_4", "4", },
        { 0xa5, "Key_5", "5", },
        { 0x65, "Key_6", "6", },
        { 0xf5, "Key_FullScreen", },

        { 0xe5, "Key_7", "7", },
        { 0x15, "Key_8", "8", },
        { 0x95, "Key_9", "9", },
        { 0xcd, "Key_Loop", "Recall", },

        { 0x55, "Key_0", "0", },
        { 0x05, "Key_Left", "Left" },
        { 0xed, "Key_Right", "Right" },
        { 0x0d, "Key_Mute", "" },

        { 0xb5, "Key_Color", ""},
        { 0x75, "Key_Preview", "" },
        { 0xdd, "Key_ChannelUp", "ChannelUp", },
        { 0x5d, "Key_VolumeUp", "VolumeUp", },

        { 0xbd, "Key_AutoScan", ""},
        { 0x9d, "Key_Enter", "Enter"},
        { 0xfd, "Key_ChannelDown", "ChannelDown", },
        { 0x7d, "Key_VolumeDown", "VolumeDown", },

};

typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;	// default first delay for a key
INT repeat_delay=100;	// default repeat delay for a key

void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
                log("Aver95.dll: Initializing Aver TVPhone (95) remote module.\n");
                log("Aver95.dll: Copyright (c) 2000 by Ivan Kolin, <ivkolin@mail.ru>.\n");
		     	log("Aver95.dll: KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		};


        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    DWORD key_pause=20; // initial repeat delay

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0x0004);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00007000));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0x7FC2);


                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
        
                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 
            			 value = ((gpio_data &0x00ff0000)>>16);
	            
                         if (gpio_data & 0x00010000) // any key is pressed
                         {
                             if (last_code == value)
                             {     // auto repeat for any pressed key
                                   key_pause = repeat_delay;
                             } 
                             else
                             {	   // first run for any pressed key 
                                   last_code = value;
                                   key_pause = first_delay;
                             }
							 AddStroke(last_code);

                         }
                         else  // no one key is pressed
                         {
                             if ((value!=last_code) && ((value & 0x1)==0) && (last_code!=0xffffffff))
                             {
                                last_code=value;
                             }
							   
                             key_pause = 20;
                         };
				
                    };

                log("Aver95: Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
